package quq.ots.example;

import org.omg.CosNaming.Binding;
import org.omg.CosNaming.BindingIteratorHolder;
import org.omg.CosNaming.BindingListHolder;
import org.omg.CosNaming.NameComponent;
import org.omg.CosNaming.NamingContextExt;
import org.omg.CosNaming.NamingContextExtHelper;
import org.omg.PortableServer.POA;
import org.omg.PortableServer.POAManager;

import quq.ots.example.util.Util;

import com.arjuna.ArjunaOTS.ArjunaFactoryHelper;
import com.arjuna.ats.internal.jdbc.recovery.BasicXARecovery;

public class Coordinator {

	quq.ots.example.resource.OracleResourceManager OracleRM;

	org.omg.CORBA.ORB orb;
	org.omg.CORBA.Object obj;
	org.omg.CosTransactions.Control control;
	org.omg.CosTransactions.Terminator terminator;
	org.omg.CosTransactions.Coordinator coordinator;
	org.omg.CosTransactions.PropagationContext propogationCtx;

	com.arjuna.ArjunaOTS.ArjunaFactory TSFactory;

	Participant1 p1;
	Participant2 p2;

	java.util.Properties props = System.getProperties();
	
	public static void main(String arg[]) {
		try {
			Coordinator c = new Coordinator();
			c.initialize();
			c.initiateTXN("vs", "vs", 10);
			c.destroy();
		} catch (Exception e) {
			System.out.println("Exception in Coordinator: " + e.getMessage());
			e.printStackTrace();
		}
	}

	void initialize() {
		try {
			System.out.println("Initializing ORB & JTS...");
			String args[] = { "-ORBInitRef.NameService=corbaname::localhost:9101" };
			Class.forName("org.jacorb.orb.ORB");
			Class.forName("org.jacorb.orb.ORBSingleton");
			props.setProperty("org.omg.CORBA.ORBClass", "org.jacorb.orb.ORB");
			props.setProperty("org.omg.CORBA.ORBSingletonClass",
					"org.jacorb.orb.ORBSingleton");
			props.setProperty("jacorb.home", Util.get_JACORB_HOME());

			orb = org.omg.CORBA.ORB.init(args, props);

			System.out.println("ORB class Name:  " + orb.getClass().getName());

			org.omg.CORBA.Object obj = orb
					.resolve_initial_references("RootPOA");
			POA poa = org.omg.PortableServer.POAHelper.narrow(obj);

			// ServantManager pman = poa.get_servant_manager();
			POAManager poaman = poa.the_POAManager();
			poaman.activate();

			org.omg.CORBA.Object nameObj = orb
					.resolve_initial_references("NameService");

			NamingContextExt rootContext = NamingContextExtHelper
					.narrow(nameObj);
			if (rootContext == null) {
				System.out.println("-----Can not find NameService");
				return;
			} else {
				System.out.println("-----Found NameService");
				System.out.println("-----Found NameService:"
						+ rootContext.toString());

			}
			org.omg.CosNaming.BindingListHolder bl = new BindingListHolder();
			org.omg.CosNaming.BindingIteratorHolder bi = new BindingIteratorHolder();
			rootContext.list(100, bl, bi);

			for (int i = 0; i < bl.value.length; i++) {
				Binding binding = bl.value[i];
				System.out.println("-----Binding Type:" + binding.binding_type);
				for (int j = 0; j < binding.binding_name.length; j++) {
					System.out.println("Binding name[" + j + "]:"
							+ binding.binding_name[j]);
				}

			}

			NameComponent name[] = new NameComponent[1];

			NameComponent name1 = new NameComponent(
					"TransactionManagerService", "OTS");
			name[0] = name1;
			org.omg.CORBA.Object tsObj = null;
			try {
				tsObj = rootContext.resolve(rootContext
						.to_name("TransactionManagerService.OTS"));
				System.out.println("-----Found TransactionManagerService:"
						+ tsObj.toString());
			} catch (Exception e) {
				e.printStackTrace();
				tsObj = rootContext.resolve(name);
			}
			// org.omg.CORBA.Object tsObj = rootContext.resolve(name);
			if (tsObj == null) {
				System.out.println("TransactionManagerService not found");
				return;
			}
			TSFactory = ArjunaFactoryHelper.narrow(tsObj);
			System.out.println("-----Found TransactionManagerService:"
					+ TSFactory.toString());

		} catch (Exception e) {
			System.out.println("Exception during Coordinator initialization: "
					+ e.getMessage());
			e.printStackTrace();
		}
	}

	void destroy() throws Exception {
		System.out.println("Destroying ORB...");
		p1.destroy();
		p2.destroy();
		orb.destroy();
		System.out.println("ORB destroyed.");
	}

	void initiateTXN(String src, String dest, float amount) {
		try {
			System.out.println("Initiating transaction...");
			control = TSFactory.create(60000);
			propogationCtx = control.get_coordinator().get_txcontext();
			System.out.println("status: "
					+ TSFactory.getCurrentStatus(propogationCtx.current.otid));
			System.out.println("Invoking Participant 1 Update...");
			p1 = new Participant1();
			String r1 = p1.updateAccount(propogationCtx, src, amount);
			System.out.println("Update 1 result: " + r1);

			System.out.println("Invoking Participant 2 Update...");
			p2 = new Participant2();
			String r2 = p2.updateAccount(propogationCtx, dest, amount);
			System.out.println("Update 2 result: " + r2);

			terminator = control.get_terminator();
//			terminator.rollback();
			terminator.commit(true);

			// TSFactory.
			// double d = Math.random();
			// double d = .1;
			// if (d < .51){
			// System.out.println("Trying to roll back transaction...");
			// try{
			// System.out.println("status: " +
			// TSFactory.getCurrentStatus(propogationCtx.current.otid));
			// terminator.rollback();
			// System.out.println("Transact192ion rolled back.");
			// }catch(Exception e){
			// System.out.println("Exception: " + e.getMessage());
			// e.printStackTrace();
			// }
			// }else{
			// System.out.println("Trying to commit transaction...");
			// try{
			// terminator.commit(false);
			//					
			// System.out.println("Transaction committed successfully.");
			// }catch(Exception e){
			// System.out.println("Exception: " + e.getMessage());
			// e.printStackTrace();
			// }
			// }

		} catch (Exception e) {
//			terminator.rollback();
			System.out.println("Exception while executing transaction: "
					+ e.getMessage());
			System.out.println("@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@"); 
			e.printStackTrace();
		}
	}

}
